@@ -1,3 +1,12 @@
+package Data::IEEE754::Tools;
+use 5.006;
+use warnings;
+use strict;
+use Carp;
+use Exporter 'import';  # just use the import() function, without the rest of the overhead of ISA
+
+use version 0.77; our $VERSION = version->declare('0.012');
+
 =pod
 
 =head1 NAME
@@ -23,9 +32,9 @@ Data::IEEE754::Tools - Various tools for understanding and manipulating the unde
 
 =head1 DESCRIPTION
 
-These tools give access to the underlying IEEE 754 floating-point representation
-used by Perl (see L<perlguts>).  They include functions for converting from the
-64bit internal representation to a string that shows those bits (either as
+These tools give access to the underlying IEEE 754 floating-point 64bit representation
+used by many instances of Perl (see L<perlguts>).  They include functions for converting
+from the 64bit internal representation to a string that shows those bits (either as
 hexadecimal or binary) and back, functions for converting that encoded value
 into a more human-readable format to give insight into the meaning of the encoded
 values, and functions to manipulate the smallest possible change for a given
@@ -76,16 +85,20 @@ interprets the encoding as a sign and a coefficient and a power of 2, or access
 the ULP and ULP-manipulating features, all using direct bit manipulation when
 appropriate.
 
-=cut
+=head2 Compatibility
 
-package Data::IEEE754::Tools;
-use 5.008005;
-use warnings;
-use strict;
-use Carp;
-use Exporter 'import';  # just use the import() function, without the rest of the overhead of ISA
+B<Data::IEEE754::Tools> works with 64bit floating-point representations.
 
-use version 0.77; our $VERSION = version->declare('v0.010');
+If you have a Perl setup which uses a larger representation (for example,
+C<use L<Config>; print $Config{nvsize}; # 16 =E<gt> 128bit>), values reported by
+this module will be reduced in precision to fit the 64bit representation.
+
+If you have a Perl setup which uses a smaller representation (for example,
+C<use L<Config>; print $Config{nvsize}; # 4 =E<gt> 32bit>), the installation
+will likely fail, because the unit tests were not set up for lower precision
+inputs.  However, forcing the installation I<might> still allow coercion
+from the smaller Perl NV into a true IEEE 754 double (64bit) floating-point,
+but there is no guarantee it will work.
 
 =head1 EXPORTABLE FUNCTIONS AND VARIABLES
 
@@ -243,7 +256,7 @@ sub to_hex_floatingpoint {
     if($exp == 1024) {
         return $sign . "0x1.#INF000000000p+0000"    if $mant eq '0000000000000';
         return $sign . "0x1.#IND000000000p+0000"    if $mant eq '8000000000000' and $sign eq '-';
-        return $sign . ( (($msb & 0x00080000) != 0x00080000) ? "0x1.#SNAN00000000p+0000" : "0x1.#QNAN00000000p+0000");
+        return $sign . ( (($msb & 0x00080000) != 0x00080000) ? "0x1.#SNAN00000000p+0000" : "0x1.#QNAN00000000p+0000");  # v0.012 coverage note: '!=' condition only triggered on systems with SNAN; ignore Devel::Cover failures on this line on systems which quiet all SNAN to QNAN
     }
     my $implied = 1;
     if( $exp == -1023 ) { # zero or denormal
@@ -264,7 +277,7 @@ sub to_dec_floatingpoint {
     if($exp == 1024) {
         return $sign . "0d1.#INF000000000000p+0000"    if $mant eq '0000000000000';
         return $sign . "0d1.#IND000000000000p+0000"    if $mant eq '8000000000000' and $sign eq '-';
-        return $sign . ( (($msb & 0x00080000) != 0x00080000) ? "0d1.#SNAN00000000000p+0000" : "0d1.#QNAN00000000000p+0000");
+        return $sign . ( (($msb & 0x00080000) != 0x00080000) ? "0d1.#SNAN00000000000p+0000" : "0d1.#QNAN00000000000p+0000");  # v0.012 coverage note: '!=' condition only triggered on systems with SNAN; ignore Devel::Cover failures on this line on systems which quiet all SNAN to QNAN
     }
     my $implied = 1;
     if( $exp == -1023 ) { # zero or denormal
@@ -357,7 +370,6 @@ sub ulp($) {
     # calculate normal ULP
     my $tog = toggle_ulp( $val );
     return abs($val - $tog);
-    return join "|", (caller(0))[3].'()', $val, $rawbin, $sign, $exp, $fract, __LINE__, '';
 }
 
 =head3 toggle_ulp( I<val> )
@@ -412,11 +424,14 @@ sub nextup {
     my ($msb,$lsb) = arr2x32b_from_double($val);
     $lsb += ($msb & 0x80000000) ? -1.0 : +1.0;
     if($lsb == 4_294_967_296.0) {
-        $lsb = 0.0;
-        $msb += ($msb & 0x80000000) ? -1.0 : +1.0;
+        $lsb  = 0.0;
+        $msb += 1.0;    # v0.012: LSB==4e9 only happens if you add one to LSB = 0xFFFFFFFF, so only when +msb; thus, remove extra check for msb sign here
     } elsif ($lsb == -1.0) {
-        $msb += ($msb & 0x80000000) ? -1.0 : +1.0;
+        $lsb  = 0xFFFFFFFF;
+        $msb -= 1.0;    # v0.012: LSB==-1  only happens if you subtract one from LSB = 0x00000000, so only when -msb; thus, remove extra check for msb sign here
     }
+    $msb &= 0xFFFFFFFF;     # v0.011_001: bugfix: ensure 32bit MSB <https://rt.cpan.org/Public/Bug/Display.html?id=116006>
+    $lsb &= 0xFFFFFFFF;     # v0.011_001: bugfix: ensure 32bit MSB <https://rt.cpan.org/Public/Bug/Display.html?id=116006>
     return hexstr754_to_double( sprintf '%08X%08X', $msb, $lsb );
 }
 
